(*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: sdl.ml,v 1.5 1999/08/21 15:23:18 filliatr Exp $ *)

exception SDL_failure of string

let _ = Callback.register_exception "SDL_failure" (SDL_failure "")


(* Initialization. *)

let flag_TIMER		= 0x00000001
let flag_AUDIO		= 0x00000010
let flag_VIDEO		= 0x00000020
let flag_CDROM		= 0x00000100
let flag_EVENTTHREAD	= 0x01000000	(* Not supported on all OS's *)
let flag_EVERYTHING	= 0x00FFFFFF

external init : int -> unit = "sdlstub_init"

external quit : unit -> unit = "sdlstub_quit"


(* Video. *)

module Video = struct

  type surface

  external free_surface : surface -> unit
    = "sdlstub_free_surface"

  external surface_width : surface -> int
    = "sdlstub_surface_width"
  external surface_height : surface -> int
    = "sdlstub_surface_height"
  external surface_flags : surface -> int
    = "sdlstub_surface_flags"
  external surface_rmask : surface -> int
    = "sdlstub_surface_rmask"
  external surface_gmask : surface -> int
    = "sdlstub_surface_gmask"
  external surface_bmask : surface -> int
    = "sdlstub_surface_bmask"
  external surface_amask : surface -> int
    = "sdlstub_surface_amask"

  let flag_SWSURFACE =  0x00000000
  let flag_HWSURFACE =	0x00000001
    
  let flag_ANYFORMAT =	0x10000000
  let flag_HWPALETTE = 	0x00000002
  let flag_DOUBLEBUF = 	0x40000000
  let flag_FULLSCREEN =	0x80000000
    
  let flag_HWACCEL =	0x00000100
  let flag_SRCCOLORKEY= 0x00001000
  let flag_RLEACCELOK =	0x00002000
  let flag_RLEACCEL =	0x00004000
  let flag_SRCALPHA =	0x00010000
  let flag_SRCCLIPPING=	0x00100000

  external must_lock : surface -> bool 
    = "sdlstub_must_lock"
  external lock_surface : surface -> unit
    = "sdlstub_lock_surface"
  external unlock_surface : surface -> unit
    = "sdlstub_unlock_surface"

  external video_mode_ok : int -> int -> int -> int -> bool 
    = "sdlstub_video_mode_ok"

  external set_video_mode : int -> int -> int -> int -> surface
    = "sdlstub_set_video_mode"

  external alloc_surface : 
    int -> int -> int -> int -> int -> int -> int -> int -> surface
    = "sdlstub_alloc_surface"

  external load_bmp : string -> surface
    = "sdlstub_load_bmp"
  external save_bmp : surface -> string -> unit
    = "sdlstub_save_bmp"
 
  external set_color_key : surface -> int -> int -> unit
    = "sdlstub_set_color_key"
  external set_alpha : surface -> int -> int -> unit
    = "sdlstub_set_alpha"
  external set_clipping : surface -> int -> int -> int -> int -> unit
    = "sdlstub_set_clipping"
  let disable_clipping s = set_clipping s 0 0 0 0

  external display_format : surface -> surface
    = "sdlstub_display_format"
  
  external map_rgb : surface -> int -> int -> int -> int
    = "sdlstub_map_rgb"

  type rect = {
    mutable rect_x : int;
    mutable rect_y : int;
    mutable rect_w : int;
    mutable rect_h : int }

  external fill_surface : surface -> int -> unit
    = "sdlstub_fill_surface"
  external fill_rect : surface -> rect -> int -> unit
    = "sdlstub_fill_rect"

  external update_surface : surface -> unit
    = "sdlstub_update_surface"
  external update_rect : surface -> int -> int -> int -> int -> unit
    = "sdlstub_update_rect"
  external update_rects : surface -> int -> rect array -> unit
    = "sdlstub_update_rects"

  external flip : surface -> unit
    = "sdlstub_flip"

  external blit_surface : surface -> rect option ->
                          surface -> rect option -> unit 
    = "sdlstub_blit_surface"

  type color = { 
    red : int;
    green : int;
    blue : int 
  }
		 
  external set_colors : surface -> color array -> int -> int -> bool
    = "sdlstub_set_colors"

  external show_cursor : bool -> unit
    = "sdlstub_show_cursor"

end


(* Events. *)

module Event  = struct

  type keysym = { 
    scancode : int;
    key : int;
    modif : int;
    unicode : int
  }

  type keystate = Pressed | Released

  type keyboard = {
    key_state : keystate;
    key_sym : keysym;
  }

  type event = 
    | Active 
    | KeyDown of keyboard
    | KeyUp of keyboard
    | MouseMotion
    | MouseButtonDown
    | MouseButtonUp
    | JoyMotion
    | JoyButtonDown
    | JoyButtonUp
    | Quit
    | SysWm
    | UnknowEvent

  external poll_event : unit -> event option
    = "sdlstub_poll_event"

  external wait_event : unit -> event
    = "sdlstub_wait_event"

end


(* Timer. *)

module Timer = struct

  external get_ticks : unit -> int
    = "sdlstub_get_ticks"

  external delay : int -> unit
    = "sdlstub_delay"

end


(* Drawing. *)

module Draw = struct

  external put_pixel : Video.surface -> int -> int -> int -> unit
    = "sdldraw_put_pixel"

  external get_pixel : Video.surface -> int -> int -> int
    = "sdldraw_get_pixel"

end

