(*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: sdl.mli,v 1.5 1999/08/21 15:23:19 filliatr Exp $ *)

exception SDL_failure of string


(* Initialization. *)

val flag_TIMER : int
val flag_AUDIO : int
val flag_VIDEO : int
val flag_CDROM : int
val flag_EVENTTHREAD : int
val flag_EVERYTHING : int

val init : int -> unit
val quit : unit -> unit


(* Video. *)

module Video : sig

  type surface

  val surface_width : surface -> int
  val surface_height : surface -> int
  val surface_flags : surface -> int
  val surface_rmask : surface -> int
  val surface_gmask : surface -> int
  val surface_bmask : surface -> int
  val surface_amask : surface -> int

  val free_surface : surface -> unit

  val flag_SWSURFACE	: int           (* Surface is in system memory *)
  val flag_HWSURFACE	: int           (* Surface is in video memory *)

  val flag_ANYFORMAT	: int           (* Allow any video pixel format *)
  val flag_HWPALETTE	: int           (* Surface has exclusive palette *)
  val flag_DOUBLEBUF	: int           (* Set up double-buffered video mode *)
  val flag_FULLSCREEN	: int           (* Surface is a full screen display *)

  val flag_HWACCEL	: int           (* Blit uses hardware acceleration *)
  val flag_SRCCOLORKEY	: int           (* Blit uses a source color key *)
  val flag_RLEACCEL	: int           (* Colorkey blit is RLE accelerated *)
  val flag_SRCALPHA	: int           (* Blit uses source alpha blending *)
  val flag_SRCCLIPPING	: int           (* Blit uses source clipping *)

  val must_lock : surface -> bool
  val lock_surface : surface -> unit
  val unlock_surface : surface -> unit

  val video_mode_ok : int -> int -> int -> int -> bool
	 (* [video_mode_ok width height bpp flags] *)
  val set_video_mode : int -> int -> int -> int -> surface
	 (* [set_video_mode width height bpp flags] *)

  val alloc_surface : int -> int -> int -> int -> int -> int -> int -> int
      -> surface
    (* [alloc_surface flags w h depth rmask gmask bmask amask] *)
  
  val load_bmp : string -> surface
  val save_bmp : surface -> string -> unit
      
  val set_color_key : surface -> int -> int -> unit
      (* [set_color_key surf flag key] *)
  val set_alpha : surface -> int -> int -> unit
      (* [set_alpha surf flag alpha] *)
  val set_clipping : surface -> int -> int -> int -> int -> unit
      (* [set_clipping surf top left bottom right] *)
  val disable_clipping : surface -> unit

  val display_format : surface -> surface

  val map_rgb : surface -> int -> int -> int -> int

  type rect = {
    mutable rect_x : int;
    mutable rect_y : int;
    mutable rect_w : int;
    mutable rect_h : int }

  val fill_surface : surface -> int -> unit
  val fill_rect : surface -> rect -> int -> unit

  val update_surface : surface -> unit
  val update_rect : surface -> int -> int -> int -> int -> unit
  val update_rects : surface -> int -> rect array -> unit

  val flip : surface -> unit

  val blit_surface : surface -> rect option -> surface -> rect option -> unit
      (* beware: rectangles are modified if clipped! *)

  type color = { 
    red : int;     (* 0..255 *)
    green : int;   (* 0..255 *)
    blue : int     (* 0..255 *)
  }
		 
  val set_colors : surface -> color array -> int -> int -> bool
      (* [set_colors surf colors firstcolor ncolors] *)

  val show_cursor : bool -> unit

end


(* Events. *)

module Event : sig

  type keysym = { 
    scancode : int;
    key : int;
    modif : int;
    unicode : int
  }

  type keystate = Pressed | Released

  type keyboard = {
    key_state : keystate;
    key_sym : keysym;
  }

  type event = 
    | Active 
    | KeyDown of keyboard
    | KeyUp of keyboard
    | MouseMotion
    | MouseButtonDown
    | MouseButtonUp
    | JoyMotion
    | JoyButtonDown
    | JoyButtonUp
    | Quit
    | SysWm
    | UnknowEvent

  val poll_event : unit -> event option

  val wait_event : unit -> event
  
end


(* Timer. *)

module Timer : sig

  val get_ticks : unit -> int

  val delay : int -> unit

end


(* Drawing. *)

module Draw : sig

  val put_pixel : Video.surface -> int -> int -> int -> unit
      
  val get_pixel : Video.surface -> int -> int -> int

end
