/*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 */

/* $Id: sdl_stub.c,v 1.5 1999/08/21 15:23:21 filliatr Exp $ */

#include <stdio.h>
#include <SDL.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/fail.h>
#include <caml/callback.h>

void raise_failure() {
  raise_with_string(*caml_named_value("SDL_failure"), SDL_GetError());
}

value sdlstub_init(value vf) {
  int flags = Int_val(vf);

  if (SDL_Init(flags) < 0) raise_failure();
  return Val_unit;
}

value sdlstub_quit(value u) {
  SDL_Quit();
  return Val_unit;
}

value sdlstub_must_lock(value s) {
  int b = SDL_MUSTLOCK(((SDL_Surface*) s));
  return Val_bool(b);
}

value sdlstub_video_mode_ok(value vw, value vh, value vbpp, value vf) {
  int w = Int_val(vw);
  int h = Int_val(vh);
  int bpp = Int_val(vbpp);
  int flags = Int_val(vf);

  return Val_bool(SDL_VideoModeOK(w,h,bpp,flags));
}

value sdlstub_set_video_mode(value vw, value vh, value vbpp, value vf) {
  int w = Int_val(vw);
  int h = Int_val(vh);
  int bpp = Int_val(vbpp);
  int flags = Int_val(vf);
  SDL_Surface* s;
  
  if ((s = SDL_SetVideoMode(w,h,bpp,flags)) == NULL) raise_failure();
  return (value) s;
}

value sdlstub_load_bmp(value vfile) {
  char * file = String_val(vfile);
  SDL_Surface* s;

  if ((s = SDL_LoadBMP(file)) == NULL) raise_failure();
  return (value) s;
}

value sdlstub_save_bmp(value s, value vfile) {
  char * file = String_val(vfile);

  if (SDL_SaveBMP((SDL_Surface*) s, file) < 0) raise_failure();
  return Val_unit;
}

value sdlstub_set_color_key(value s, value vf, value vk) {
  int flag = Int_val(vf);
  int key = Int_val(vk);

  if (SDL_SetColorKey((SDL_Surface*) s, flag, key) < 0) raise_failure();
  return Val_unit;
}

value sdlstub_set_alpha(value s, value vf, value va) {
  int flags = Int_val(vf);
  int alpha = Int_val(va);

  if (SDL_SetAlpha((SDL_Surface*) s, flags, alpha) < 0) raise_failure();
  return Val_unit;
}

value sdlstub_set_clipping(value s, value vtop, value vleft, 
			   value vbottom, value vright) {
  int top = Int_val(vtop);
  int left = Int_val(vleft);
  int bottom = Int_val(vbottom);
  int right = Int_val(vright);
  
  SDL_SetClipping((SDL_Surface*) s, top, left, bottom, right);
  return Val_unit;
}

value sdlstub_display_format(value s) {
  SDL_Surface* n;

  if ((n = SDL_DisplayFormat((SDL_Surface*) s)) == NULL) raise_failure();
  return (value) n;
}

value sdlstub_alloc_surface(value vflags, value vw, value vh, value vdepth, 
			    value vrmask, value vgmask, value vbmask,
			    value vamask) {
  SDL_Surface* s;
  Uint32 flags = Int_val(vflags);
  int width = Int_val(vw);
  int height = Int_val(vh);
  int depth = Int_val(vdepth);
  Uint32 rmask = Int_val(vrmask);
  Uint32 gmask = Int_val(vgmask);
  Uint32 bmask = Int_val(vbmask);
  Uint32 amask = Int_val(vamask);
  
  s = SDL_AllocSurface(flags, width, height, depth, 
		       rmask, gmask, bmask, amask); 
  if (s == NULL) raise_failure(); 
  return (value) s;
}

value sdlstub_free_surface(value s) {
  SDL_FreeSurface((SDL_Surface*) s);
  return Val_unit;
}

value sdlstub_lock_surface(value s) {
  if (SDL_LockSurface((SDL_Surface*) s) < 0) raise_failure();
  return Val_unit;
}

value sdlstub_unlock_surface(value s) {
  SDL_UnlockSurface((SDL_Surface*) s);
  return Val_unit;
}

value sdlstub_surface_width(value s) {
  return Val_int(((SDL_Surface*) s)->w);
}

value sdlstub_surface_height(value s) {
  return Val_int(((SDL_Surface*) s)->h);
}

value sdlstub_surface_flags(value s) {
  return Val_int(((SDL_Surface*) s)->flags);
}

value sdlstub_surface_rmask(value s) {
  return Val_int(((SDL_Surface*) s)->format->Rmask);
}

value sdlstub_surface_gmask(value s) {
  return Val_int(((SDL_Surface*) s)->format->Gmask);
}

value sdlstub_surface_bmask(value s) {
  return Val_int(((SDL_Surface*) s)->format->Bmask);
}

value sdlstub_surface_amask(value s) {
  return Val_int(((SDL_Surface*) s)->format->Amask);
}

value sdlstub_map_rgb(value s, value vr, value vg, value vb) {
  int r = Int_val(vr);
  int g = Int_val(vg);
  int b = Int_val(vb);

  return Val_int(SDL_MapRGB(((SDL_Surface*) s)->format, r, g, b));
}

value sdlstub_get_ticks(value u) {
  return Val_int(SDL_GetTicks());
}

value sdlstub_delay(value vms) {
  int ms = Int_val(vms);

  SDL_Delay(ms);
  return Val_unit;
}

value sdlstub_fill_surface(value s, value vc) {
  int c = Int_val(vc);

  if (SDL_FillRect((SDL_Surface*) s, NULL, c) < 0) raise_failure();
  return Val_unit;
}

value sdlstub_fill_rect(value s, value vr, value vc) {
  int c = Int_val(vc);
  SDL_Rect r;
  
  r.x = Int_val(Field(vr,0));
  r.y = Int_val(Field(vr,1));
  r.w = Int_val(Field(vr,2));
  r.h = Int_val(Field(vr,3));
  if (SDL_FillRect((SDL_Surface*) s, &r, c) < 0) raise_failure();
  return Val_unit;
}

value sdlstub_update_surface(value s) {
  SDL_UpdateRect((SDL_Surface*) s, 0, 0, 0, 0);
  return Val_unit;
}

value sdlstub_update_rect(value s, value vx, value vy, value vw, value vh) {
  int x = Int_val(vx);
  int y = Int_val(vy);
  int w = Int_val(vw);
  int h = Int_val(vh);
  
  SDL_UpdateRect((SDL_Surface*) s, x, y, w, h);
  return Val_unit;
}

value sdlstub_update_rects(value s, value vn, value arr){
  int n = Int_val(vn);
  value v;
  int i;
  SDL_Rect* rects;

  if (Wosize_val(arr) < n) invalid_argument("update_rects");
  rects = (SDL_Rect*) malloc(n * sizeof(SDL_Rect));
  for (i = 0; i < n; i++) {
    v = Field(arr,i);
    rects[i].x = Int_val(Field(v,0));
    rects[i].y = Int_val(Field(v,1));
    rects[i].w = Int_val(Field(v,2));
    rects[i].h = Int_val(Field(v,3));
  };
  SDL_UpdateRects((SDL_Surface*) s, n, rects);
  free(rects);
  return Val_unit;
}

value sdlstub_flip(value s) {
  if (SDL_Flip((SDL_Surface*) s) < 0) raise_failure();
  return Val_unit;
}

SDL_Rect* rect_from_option(value v, SDL_Rect* r) {
  value vr;

  if (v == Val_int(0)) { 
    /* None */
    return NULL;
  } else {
    /* Some */
    vr = Field(v,0);
    r->x = Int_val(Field(vr,0));
    r->y = Int_val(Field(vr,1));
    r->w = Int_val(Field(vr,2));
    r->h = Int_val(Field(vr,3));
    return r;
  }
}

/* assumption: v = Some rect  and  r is not NULL */
void update_rect_option(value v, SDL_Rect* r) {
  value vr = Field(v,0);
  modify(&Field(vr,0), Val_int(r->x));
  modify(&Field(vr,1), Val_int(r->y));
  modify(&Field(vr,2), Val_int(r->w));
  modify(&Field(vr,3), Val_int(r->h));
}

value sdlstub_blit_surface(value src, value srcr, value dst, value dstr) {
  SDL_Rect sr,dr;
  SDL_Rect *srp, *drp;

  srp = rect_from_option(srcr,&sr);
  drp = rect_from_option(dstr,&dr);
  if (SDL_BlitSurface((SDL_Surface*) src, srp, (SDL_Surface*) dst, drp) < 0)
    raise_failure();
  if (! (srp == NULL)) update_rect_option(srcr,srp);
  if (! (drp == NULL)) update_rect_option(dstr,drp);
  return Val_unit;
}

value value_for_keysym(SDL_Event* event) {
  value v = Val_unit;
  SDL_keysym ks = event->key.keysym;

  Begin_root(v);
    v = alloc_tuple(4);
    Field(v,0) = Val_int(ks.scancode);
    Field(v,1) = Val_int(ks.sym);
    Field(v,2) = Val_int(ks.mod);
    Field(v,3) = Val_int(ks.unicode);
  End_roots();
  return v;
}

value value_for_keyboard(SDL_Event* event) {
  value v = Val_unit;
  value ks;

  ks = value_for_keysym(event);
  Begin_root(v);
    v = alloc_tuple(2);
    Field(v,0) = Val_int((event->key.state == SDL_PRESSED) ? 0 : 1);
    Field(v,1) = ks;
  End_roots();
  return v;
}

/* returns (C_tag v) */
value unary_constructor(value v, int tag) {
  value result = Val_unit;

  Begin_root(result);
    result = alloc(1,tag);
    Field(result,0) = v;
  End_roots();
  return result;
}

value value_for_event(SDL_Event* event) {
  int tag;
  value v;

  switch (event->type) {
  case SDL_ACTIVEEVENT:
    return Val_int(0);
  case SDL_KEYDOWN:
  case SDL_KEYUP:
    v = value_for_keyboard(event);
    tag = ((event->type == SDL_KEYDOWN) ? 0 : 1);
    return unary_constructor(v,tag);
  case SDL_QUIT:
    return Val_int(7);
  default:
    return Val_int(9);
  }
}

value sdlstub_poll_event(value u) {
  SDL_Event event;
  value v;
  value result = Val_unit;

  if (SDL_PollEvent(&event)) {
    /* Some */
    v = value_for_event(&event);
    Begin_root(result);
      result = alloc(1,0);
      Field(result,0) = v;
    End_roots();
    return result;
  } else { 
    /* None */
    return Val_int(0);
  }
}

value sdlstub_wait_event(value u) {
  SDL_Event event;
  value v;

  if (SDL_WaitEvent(&event)) {
    return value_for_event(&event);
  } else { 
    raise_failure();
  }
}

value sdlstub_set_colors(value s, value arr, value vfirst, value vn) {
  int ncolors = Int_val(vn);
  int firstcolor = Int_val(vfirst);
  int result, i;
  value v;
  SDL_Color* colors;

  if (ncolors < 0 || ncolors > Wosize_val(arr)) invalid_argument("set_colors");
  colors = (SDL_Color*) malloc(ncolors * sizeof(SDL_Color));
  if (colors == NULL) raise_out_of_memory();
  for (i=0; i<ncolors; i++) {
    v = Field(arr,i);
    colors[i].r = Int_val(Field(v,0));
    colors[i].g = Int_val(Field(v,1));
    colors[i].b = Int_val(Field(v,2));
  };
  result = SDL_SetColors((SDL_Surface*) s, colors, firstcolor, ncolors);
  free(colors);
  return Val_bool(result);
}

value sdlstub_show_cursor(value vtoggle) {
  int toggle = Bool_val(vtoggle);
  
  SDL_ShowCursor(toggle);
  return Val_unit;
}
